@@ -49,12 +49,13 @@
 #include <errno.h>
 #endif
 
+#include <crc32.h>
 #include <uudeview.h>
 #include <uuint.h>
 #include <fptools.h>
 #include <uustring.h>
 
-char * uunconc_id = "$Id: uunconc.c,v 1.6 2002/03/31 22:11:58 root Exp $";
+char * uunconc_id = "$Id: uunconc.c,v 1.11 2002/10/13 13:08:44 root Exp $";
 
 /* for braindead systems */
 #ifndef SEEK_SET
@@ -139,8 +140,8 @@ UUInitConc (void)
   BHxlat  = (int *) uunconc_BHxlat;
 
   save[0] = uunconc_save;
-  save[1] = uunconc_save + 256;
-  save[2] = uunconc_save + 512;
+  save[1] = uunconc_save + 1200;
+  save[2] = uunconc_save + 2400;
 
   /* prepare decoding translation table */
   for(i = 0; i < 256; i++)
@@ -544,7 +545,7 @@ UURepairData (FILE *datei, char *line, int encoding, int *bhflag)
       ptr = line + strlen (line);
       while (ptr>line && (*(ptr-1)=='\015' || *(ptr-1)=='\012'))
 	ptr--;
-      if (_FP_fgets (ptr, 255-(ptr-line), datei) == NULL)
+      if (_FP_fgets (ptr, 299-(ptr-line), datei) == NULL)
 	break;
     }
     else {			/* don't need next line to repair */
@@ -649,7 +650,8 @@ UUDecodeLine (char *s, char *d, int method)
   }
   else if (method == B64ENCODED) {
     if (leftover) {
-      strcpy (uuncdl_fulline+leftover, s);
+      strcpy (uuncdl_fulline + leftover, s);
+
       leftover = 0;
       s        = uuncdl_fulline;
     }
@@ -679,7 +681,8 @@ UUDecodeLine (char *s, char *d, int method)
   }
   else if (method == BH_ENCODED) {
     if (leftover) {
-      strcpy (uuncdl_fulline+leftover, s);
+      strcpy (uuncdl_fulline + leftover, s);
+
       leftover = 0;
       s        = uuncdl_fulline;
     }
@@ -717,7 +720,7 @@ UUDecodeLine (char *s, char *d, int method)
 	  s++;
 	}
       }
-      else if (*s == '\t' || *s == '\n' || *s == '\r') {
+      else if (*s == '\n' || *s == '\r') {
 	s++; /* ignore */
       }
       else {
@@ -746,7 +749,7 @@ UUDecodeQP (FILE *datain, FILE *dataout, int *state,
   while (!feof (datain) && 
 	 (ftell(datain)<maxpos || flags&FL_TOEND ||
 	  (!(flags&FL_PROPER) && uu_fast_scanning))) {
-    if (_FP_fgets (line, 255, datain) == NULL)
+    if (_FP_fgets (line, 1023, datain) == NULL)
       break;
     if (ferror (datain)) {
       UUMessage (uunconc_id, __LINE__, UUMSG_ERROR,
@@ -843,7 +846,7 @@ UUDecodePT (FILE *datain, FILE *dataout, int *state,
   while (!feof (datain) && 
 	 (ftell(datain)<maxpos || flags&FL_TOEND ||
 	  (!(flags&FL_PROPER) && uu_fast_scanning))) {
-    if (_FP_fgets (line, 255, datain) == NULL)
+    if (_FP_fgets (line, 1023, datain) == NULL)
       break;
     if (ferror (datain)) {
       UUMessage (uunconc_id, __LINE__, UUMSG_ERROR,
@@ -899,15 +902,19 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	      long maxpos, int method, int flags,
 	      char *boundary)
 {
-  char *line=uugen_fnbuffer, *oline=uuncdp_oline;
+  char *line, *oline=uuncdp_oline;
   int warning=0, vlc=0, lc[2], hadct=0;
   int tc=0, tf=0, vflag, haddata=0, haddh=0;
   long yefilesize=0, yepartends=0;
+  crc32_t yepartcrc=crc32(0L, Z_NULL, 0);
+  static crc32_t yefilecrc=0;
   static int bhflag=0;
   size_t count=0;
+  size_t yepartsize=0;
   char *ptr;
 
   if (datain == NULL || dataout == NULL) {
+    yefilecrc = crc32(0L, Z_NULL, 0);
     bhflag = 0;
     return UURET_OK;
   }
@@ -935,9 +942,13 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
   while (!feof (datain) && *state != DONE && 
 	 (ftell(datain)<maxpos || flags&FL_TOEND || maxpos==-1 ||
 	  (!(flags&FL_PROPER) && uu_fast_scanning))) {
-    if (_FP_fgets (line, 299, datain) == NULL)
+    if (_FP_fgets ((line = uugen_fnbuffer), 1200 - 5, datain) == NULL)
       break;
 
+    /* optionally skip .. */
+    if (*line == '.' && uu_dotdot)
+      line++;
+
     if (ferror (datain)) {
       UUMessage (uunconc_id, __LINE__, UUMSG_ERROR,
 		 uustring (S_SOURCE_READ_ERR),
@@ -977,8 +988,8 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
      * try to make sense of data
      */
 
-    line[299] = '\0'; /* For Safety of string functions */
-    count     =  0;
+    line[1200 - 1] = '\0'; /* For Safety of string functions */
+    count          =  0;
 
     if (boundary && line[0]=='-' && line[1]=='-' &&
 	strncmp (line+2, boundary, strlen (boundary)) == 0) {
@@ -1032,15 +1043,19 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
       }
       else if (method == YENC_ENCODED &&
 	       strncmp (line, "=ybegin ", 8) == 0 &&
-	       _FP_strstr (line, " size=") != NULL &&
 	       _FP_strstr (line, " name=") != NULL) {
 	*state = DATA;
 
-	ptr = _FP_strstr (line, " size=") + 6;
-	yefilesize = atoi (ptr);
+	if ((ptr = _FP_strstr (line, " size=")) != NULL) {
+	  ptr += 6;
+	  yefilesize = atoi (ptr);
+	}
+	else {
+	  yefilesize = -1;
+	}
 
 	if (_FP_strstr (line, " part=") != NULL) {
-	  if (_FP_fgets (line, 299, datain) == NULL) {
+	  if (_FP_fgets (line, 1200 - 5, datain) == NULL) {
 	    break;
 	  }
 
@@ -1070,6 +1085,35 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 
     if (*state == DATA && method == YENC_ENCODED &&
 	strncmp (line, "=yend ", 6) == 0) {
+      if ((ptr = _FP_strstr (line, " pcrc32=")) != NULL) {
+	crc32_t pcrc32 = strtoul (ptr + 8, NULL, 16);
+	if (pcrc32 != yepartcrc) {
+	  UUMessage (uunconc_id, __LINE__, UUMSG_WARNING,
+		     uustring (S_PCRC_MISMATCH), progress.curfile, progress.partno);
+	}
+      }
+      if ((ptr = _FP_strstr (line, " crc32=")) != NULL)
+      {
+	crc32_t fcrc32 = strtoul (ptr + 7, NULL, 16);
+	if (fcrc32 != yefilecrc) {
+	  UUMessage (uunconc_id, __LINE__, UUMSG_WARNING,
+		     uustring (S_CRC_MISMATCH), progress.curfile);
+	}
+      }
+      if ((ptr = _FP_strstr (line, " size=")) != NULL)
+      {
+	size_t size = atol(ptr + 6);
+	if (size != yepartsize && yefilesize != -1) {
+	  if (size != yefilesize)
+	    UUMessage (uunconc_id, __LINE__, UUMSG_WARNING,
+		       uustring (S_PSIZE_MISMATCH), progress.curfile,
+		       progress.partno, yepartsize, size);
+	  else
+	    UUMessage (uunconc_id, __LINE__, UUMSG_WARNING,
+		       uustring (S_SIZE_MISMATCH), progress.curfile,
+		       yepartsize, size);
+	}
+      }
       if (yepartends == 0 || yepartends >= yefilesize) {
 	*state = DONE;
       }
@@ -1097,6 +1141,12 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
       if (vflag == method) {
 	if (tf) {
 	  count  = UUDecodeLine (line, oline, method);
+	  if (method == YENC_ENCODED) {
+	    if (yepartends)
+	      yepartcrc = crc32(yepartcrc, oline, count);
+	    yefilecrc = crc32(yefilecrc, oline, count);
+	    yepartsize += count;
+	  }
 	  vlc++; lc[1]++;
 	}
 	else if (tc == 3) {
@@ -1122,7 +1172,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	  lc[1] = 3;
 	}
 	else {
-	  _FP_strncpy (save[tc++], line, 256);
+	  _FP_strncpy (save[tc++], line, 1200);
 	}
 
 	if (method == UU_ENCODED)
